"""
This type stub file was generated by pyright.
"""

from typing import Any, Callable, override
from pyjudo.core import IResolver, IScopeStack, IServiceCache, IServiceScope

class ServiceScope(IServiceScope):
    """
    Represents a scope for services.
    """
    def __init__(self, cache: IServiceCache, stack: IScopeStack, resolver: IResolver) -> None:
        ...
    
    @override
    def get[T](self, interface: type[T], **overrides: Any) -> T:
        ...
    
    @override
    def get_instance[T](self, interface: type[T]) -> T | None:
        ...
    
    @override
    def add_instance[T](self, interface: type[T], instance: T) -> None:
        ...
    
    @override
    def __getitem__[T](self, key: type[T]) -> Callable[..., T]:
        ...
    
    @override
    def __enter__(self): # -> Self:
        ...
    
    @override
    def __exit__(self, _exc_type, _exc_value, _traceback): # -> None:
        ...
    


