"""
This type stub file was generated by pyright.
"""

from collections.abc import Callable
from .service_life import ServiceLife

class ServiceEntry[T]:
    """
    Represents a service entry in the container.
    """
    __slots__ = ...
    constructor: type[T] | Callable[..., T]
    lifetime: ServiceLife
    def __init__(self, constructor: type[T] | Callable[..., T], lifetime: ServiceLife) -> None:
        ...
    


