"""
This type stub file was generated by pyright.
"""

from typing import Any, Callable, override
from pyjudo.core import IResolver, IScopeStack, IServiceCache, IServiceEntryCollection

class Resolver(IResolver):
    """
    Resolves services from the container.
    The resolver is responsible for retrieving service instances from a container.
    """
    def __init__(self, service_entry_collection: IServiceEntryCollection, singleton_cache: IServiceCache, scope_stack: IScopeStack) -> None:
        ...
    
    @override
    def resolve_anonymous[T](self, constructor: Callable[..., T], overrides: dict[str, Any], binding: Any | None = ...) -> T:
        ...
    
    @override
    def resolve[T](self, interface: type[T], overrides: dict[str, Any]) -> T:
        ...
    


