"""
This type stub file was generated by pyright.
"""

from .iresolver import IResolver
from .iservice_cache import IServiceCache
from .iservice_container import IServiceContainer
from .iservice_entry_collection import IServiceEntryCollection
from .iservice_scope import IServiceScope
from .iservice_stack import IScopeStack
from .service_life import ServiceLife
from .service_entry import ServiceEntry

__all__ = ["IResolver", "IServiceCache", "IServiceContainer", "IServiceEntryCollection", "IServiceScope", "IScopeStack", "ServiceLife", "ServiceEntry"]
